.g-current-progress-message, .g-overall-progress-message
  color #737373
  margin-top 5px
  margin-bottom 5px

.g-progress-current, .g-progress-overall
  >.progress-bar
    transition none

.g-drop-zone
  padding 11px 22px 11px 17px
  font-size 16px
  font-weight bold
  color white
  background-color #5cb85c
  border 1px solid #595
  border-radius 6px
  display inline-block
  margin-bottom 10px

.g-drop-zone:hover
  cursor pointer
  border 1px solid #484
  background-color #4ca84c
  background-image linear-gradient(to bottom, #4ca84c 0, #5cb85c 100%)

.g-dropzone-show
  border-radius 5px
  border 2px dashed #bbb
  background-color white
  color #999
  font-size 18px
  width 100%
  padding 18px 30px
  text-align center

  i
    pointer-events none

.g-nonmodal-upload-buttons-container
  margin-top 15px
