module.exports = {
    disallowAttributeInterpolation: true,
    disallowClassAttributeWithStaticValue: true,
    disallowClassLiteralsBeforeIdLiterals: true,
    disallowDuplicateAttributes: true,
    disallowHtmlText: true,
    disallowIdAttributeWithStaticValue: true,
    disallowLegacyMixinCall: true,
    disallowMultipleLineBreaks: true,
    disallowSpacesInsideAttributeBrackets: true,
    disallowStringConcatenation: 'aggressive',
    disallowTrailingSpaces: true,
    requireClassLiteralsBeforeAttributes: true,
    requireIdLiteralsBeforeAttributes: true,
    requireLineFeedAtFileEnd: true,
    requireLowerCaseAttributes: true,
    requireLowerCaseTags: true,
    requireSpaceAfterCodeOperator: true,
    requireStrictEqualityOperators: true,
    validateAttributeSeparator: {
        separator: ', ',
        multiLineSeparator: ',\n    '
    },
    validateDivTags: true,
    validateIndentation: 2,
    validateLineBreaks: 'LF',
    validateSelfClosingTags: true,
    validateTemplateString: true
};
